@@ -38,6 +38,7 @@
 #include <QFile>
 #include <QSettings>
 #include <QStandardItemModel>
+#include <QSortFilterProxyModel>
 #include <QHeaderView>
 #include <QMenu>
 #include <QMessageBox>
@@ -46,13 +47,24 @@ FinishedTorrents::FinishedTorrents(QObject *parent, bittorrent *BTSession) : par
   setupUi(this);
   actionStart->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/play.png")));
   actionPause->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/pause.png")));
-  finishedListModel = new QStandardItemModel(0,6);
+  finishedListModel = new QStandardItemModel(0,8);
   finishedListModel->setHeaderData(F_NAME, Qt::Horizontal, tr("Name", "i.e: file name"));
   finishedListModel->setHeaderData(F_SIZE, Qt::Horizontal, tr("Size", "i.e: file size"));
   finishedListModel->setHeaderData(F_UPSPEED, Qt::Horizontal, tr("UP Speed", "i.e: Upload speed"));
-  finishedListModel->setHeaderData(F_LEECH, Qt::Horizontal, tr("Leechers", "i.e: full/partial sources"));
+  finishedListModel->setHeaderData(F_SWARM, Qt::Horizontal, tr("Seeds / Leechers"));
+  finishedListModel->setHeaderData(F_PEERS, Qt::Horizontal, tr("Connected peers"));
+  finishedListModel->setHeaderData(F_UPLOAD, Qt::Horizontal, tr("Total uploaded", "i.e: Total amount of uploaded data"));
   finishedListModel->setHeaderData(F_RATIO, Qt::Horizontal, tr("Ratio"));
-  finishedList->setModel(finishedListModel);
+
+  proxyModel = new QSortFilterProxyModel();
+  proxyModel->setDynamicSortFilter(true);
+  proxyModel->setSourceModel(finishedListModel);
+  finishedList->setModel(proxyModel);
+
+  finishedList->setRootIsDecorated(false);
+  finishedList->setAllColumnsShowFocus(true);
+  finishedList->setSortingEnabled(true);
+
   loadHiddenColumns();
   // Hide hash column
   finishedList->hideColumn(F_HASH);
@@ -60,10 +72,11 @@ FinishedTorrents::FinishedTorrents(QObject *parent, bittorrent *BTSession) : par
   if(!loadColWidthFinishedList()){
     finishedList->header()->resizeSection(0, 200);
   }
+  // Connect BTSession signals
+  connect(BTSession, SIGNAL(metadataReceived(QTorrentHandle&)), this, SLOT(updateMetadata(QTorrentHandle&)));
   // Make download list header clickable for sorting
   finishedList->header()->setClickable(true);
   finishedList->header()->setSortIndicatorShown(true);
-  connect(finishedList->header(), SIGNAL(sectionPressed(int)), this, SLOT(toggleFinishedListSortOrder(int)));
   finishedListDelegate = new FinishedListDelegate(finishedList);
   finishedList->setItemDelegate(finishedListDelegate);
   connect(finishedList, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayFinishedListMenu(const QPoint&)));
@@ -75,6 +88,8 @@ FinishedTorrents::FinishedTorrents(QObject *parent, bittorrent *BTSession) : par
   actionDelete_Permanently->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/delete_perm.png")));
   actionTorrent_Properties->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/properties.png")));
   actionSet_upload_limit->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/seeding.png")));
+  actionCopy_magnet_link->setIcon(QIcon(QString::fromUtf8(":/Icons/magnet.png")));
+
   connect(actionPause, SIGNAL(triggered()), (GUI*)parent, SLOT(on_actionPause_triggered()));
   connect(actionStart, SIGNAL(triggered()), (GUI*)parent, SLOT(on_actionStart_triggered()));
   connect(actionDelete, SIGNAL(triggered()), (GUI*)parent, SLOT(on_actionDelete_triggered()));
@@ -84,18 +99,23 @@ FinishedTorrents::FinishedTorrents(QObject *parent, bittorrent *BTSession) : par
   connect(actionBuy_it, SIGNAL(triggered()), (GUI*)parent, SLOT(goBuyPage()));
   connect(actionTorrent_Properties, SIGNAL(triggered()), this, SLOT(propertiesSelection()));
   connect(actionForce_recheck, SIGNAL(triggered()), this, SLOT(forceRecheck()));
+  connect(actionCopy_magnet_link, SIGNAL(triggered()), (GUI*)parent, SLOT(copyMagnetURI()));
 
   connect(actionHOSColName, SIGNAL(triggered()), this, SLOT(hideOrShowColumnName()));
   connect(actionHOSColSize, SIGNAL(triggered()), this, SLOT(hideOrShowColumnSize()));
   connect(actionHOSColUpSpeed, SIGNAL(triggered()), this, SLOT(hideOrShowColumnUpSpeed()));
-  connect(actionHOSColLeechers, SIGNAL(triggered()), this, SLOT(hideOrShowColumnLeechers()));
+  connect(actionHOSColSwarm, SIGNAL(triggered()), this, SLOT(hideOrShowColumnSwarm()));
+  connect(actionHOSColPeers, SIGNAL(triggered()), this, SLOT(hideOrShowColumnPeers()));
+  connect(actionHOSColUpload, SIGNAL(triggered()), this, SLOT(hideOrShowColumnUpload()));
   connect(actionHOSColRatio, SIGNAL(triggered()), this, SLOT(hideOrShowColumnRatio()));
 }
 
 FinishedTorrents::~FinishedTorrents(){
+  saveLastSortedColumn();
   saveColWidthFinishedList();
   saveHiddenColumns();
   delete finishedListDelegate;
+  delete proxyModel;
   delete finishedListModel;
 }
 
@@ -115,7 +135,9 @@ void FinishedTorrents::addTorrent(QString hash){
   finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(h.name()));
   finishedListModel->setData(finishedListModel->index(row, F_SIZE), QVariant((qlonglong)h.actual_size()));
   finishedListModel->setData(finishedListModel->index(row, F_UPSPEED), QVariant((double)0.));
-  finishedListModel->setData(finishedListModel->index(row, F_LEECH), QVariant("0"));
+  finishedListModel->setData(finishedListModel->index(row, F_SWARM), QVariant("-1/-1"));
+  finishedListModel->setData(finishedListModel->index(row, F_PEERS), QVariant("0"));
+  finishedListModel->setData(finishedListModel->index(row, F_UPLOAD), QVariant((qlonglong)h.all_time_upload()));
   finishedListModel->setData(finishedListModel->index(row, F_RATIO), QVariant(QString::fromUtf8(misc::toString(BTSession->getRealRatio(hash)).c_str())));
   finishedListModel->setData(finishedListModel->index(row, F_HASH), QVariant(hash));
   if(h.is_paused()) {
@@ -128,8 +150,8 @@ void FinishedTorrents::addTorrent(QString hash){
   // Update the number of finished torrents
   ++nbFinished;
   emit finishedTorrentsNumberChanged(nbFinished);
-  // Sort List
-  sortFinishedList();
+
+  loadLastSortedColumn();
 }
 
 // Set the color of a row in data model
@@ -146,7 +168,7 @@ QStringList FinishedTorrents::getSelectedTorrents(bool only_one) const{
   foreach(const QModelIndex &index, selectedIndexes) {
     if(index.column() == F_NAME) {
       // Get the file hash
-      QString hash = finishedListModel->data(finishedListModel->index(index.row(), F_HASH)).toString();
+      QString hash = getHashFromRow(index.row());
       res << hash;
       if(only_one) break;
     }
@@ -174,10 +196,39 @@ bool FinishedTorrents::loadColWidthFinishedList(){
         finishedList->header()->resizeSection(i, width_list.at(i).toInt());
   }
   loadLastSortedColumn();
+  QVariantList visualIndexes = settings.value(QString::fromUtf8("FinishedListVisualIndexes"), QVariantList()).toList();
+  if(visualIndexes.size() != finishedListModel->columnCount()-1) {
+    qDebug("Corrupted values for download list columns sizes");
+    return false;
+  }
+  bool change = false;
+  do {
+    change = false;
+    for(int i=0;i<visualIndexes.size(); ++i) {
+      int new_visual_index = visualIndexes.at(finishedList->header()->logicalIndex(i)).toInt();
+      if(i != new_visual_index) {
+        qDebug("Moving column from %d to %d", finishedList->header()->logicalIndex(i), new_visual_index);
+        finishedList->header()->moveSection(i, new_visual_index);
+        change = true;
+      }
+    }
+  }while(change);
   qDebug("Finished list columns width loaded");
   return true;
 }
 
+void FinishedTorrents::saveLastSortedColumn() {
+  QSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  Qt::SortOrder sortOrder = finishedList->header()->sortIndicatorOrder();
+  QString sortOrderLetter;
+  if(sortOrder == Qt::AscendingOrder)
+    sortOrderLetter = QString::fromUtf8("a");
+  else
+    sortOrderLetter = QString::fromUtf8("d");
+  int index = finishedList->header()->sortIndicatorSection();
+  settings.setValue(QString::fromUtf8("FinishedListSortedCol"), misc::toQString(index)+sortOrderLetter);
+}
+
 void FinishedTorrents::loadLastSortedColumn() {
   // Loading last sorted column
   QSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
@@ -190,7 +241,7 @@ void FinishedTorrents::loadLastSortedColumn() {
       sortOrder = Qt::AscendingOrder;
     sortedCol = sortedCol.left(sortedCol.size()-1);
     int index = sortedCol.toInt();
-    sortFinishedList(index, sortOrder);
+    finishedList->sortByColumn(index, sortOrder);
   }
 }
 
@@ -208,7 +259,7 @@ void FinishedTorrents::saveColWidthFinishedList() const{
     width_list = line.split(' ');
   }
   for(short i=0; i<nbColumns; ++i){
-    if(finishedList->columnWidth(i)<1 && width_list.size() == finishedListModel->columnCount()-1 && width_list.at(i).toInt()>=1) {
+    if(finishedList->columnWidth(i)<1 && width_list.size() == nbColumns && width_list.at(i).toInt()>=1) {
       // load the former width
       new_width_list << width_list.at(i);
     } else if(finishedList->columnWidth(i)>=1) { 
@@ -221,6 +272,11 @@ void FinishedTorrents::saveColWidthFinishedList() const{
     }
   }
   settings.setValue("FinishedListColsWidth", new_width_list.join(" "));
+  QVariantList visualIndexes;
+  for(int i=0; i<nbColumns; ++i) {
+    visualIndexes.append(finishedList->header()->visualIndex(i));
+  }
+  settings.setValue(QString::fromUtf8("FinishedListVisualIndexes"), visualIndexes);
   qDebug("Finished list columns width saved");
 }
 
@@ -230,13 +286,24 @@ void FinishedTorrents::on_actionSet_upload_limit_triggered(){
   foreach(const QModelIndex &index, selectedIndexes){
     if(index.column() == F_NAME){
       // Get the file hash
-      hashes << finishedListModel->data(finishedListModel->index(index.row(), F_HASH)).toString();
+      hashes << getHashFromRow(index.row());
     }
   }
   new BandwidthAllocationDialog(this, true, BTSession, hashes);
 }
 
+void FinishedTorrents::updateMetadata(QTorrentHandle &h) {
+  QString hash = h.hash();
+  int row = getRowFromHash(hash);
+  if(row != -1) {
+    qDebug("Updating torrent metadata in download list");
+    finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(h.name()));
+    finishedListModel->setData(finishedListModel->index(row, F_SIZE), QVariant((qlonglong)h.actual_size()));
+  }
+}
+
 void FinishedTorrents::updateTorrent(QTorrentHandle h) {
+    if(!h.is_valid()) return;
     QString hash = h.hash();
     int row = getRowFromHash(hash);
     if(row == -1){
@@ -245,22 +312,25 @@ void FinishedTorrents::updateTorrent(QTorrentHandle h) {
       row = getRowFromHash(hash);
     }
     Q_ASSERT(row != -1);
+    if(!finishedList->isColumnHidden(F_SWARM)) {
+      finishedListModel->setData(finishedListModel->index(row, F_SWARM), misc::toQString(h.num_complete())+QString("/")+misc::toQString(h.num_incomplete()));
+    }
     if(h.is_paused()) return;
     // Update queued torrent
     if(BTSession->isQueueingEnabled() && h.is_queued()) {
         if(h.state() == torrent_status::checking_files || h.state() == torrent_status::queued_for_checking){
-            finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(QIcon(QString::fromUtf8(":/Icons/time.png"))), Qt::DecorationRole);
+            finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(QIcon(QString::fromUtf8(":/Icons/oxygen/time.png"))), Qt::DecorationRole);
         } else {
             finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(QIcon(QString::fromUtf8(":/Icons/skin/queued.png"))), Qt::DecorationRole);
         }
         // Reset upload speed and seeds/leech
         finishedListModel->setData(finishedListModel->index(row, F_UPSPEED), 0.);
-        finishedListModel->setData(finishedListModel->index(row, F_LEECH), "0");
+        finishedListModel->setData(finishedListModel->index(row, F_PEERS), "0");
         setRowColor(row, QString::fromUtf8("grey"));
         return;
     }
     if(h.state() == torrent_status::checking_files || h.state() == torrent_status::queued_for_checking){
-      finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(QIcon(QString::fromUtf8(":/Icons/time.png"))), Qt::DecorationRole);
+      finishedListModel->setData(finishedListModel->index(row, F_NAME), QVariant(QIcon(QString::fromUtf8(":/Icons/oxygen/time.png"))), Qt::DecorationRole);
       setRowColor(row, QString::fromUtf8("grey"));
       return;
     }
@@ -269,8 +339,11 @@ void FinishedTorrents::updateTorrent(QTorrentHandle h) {
     if(!finishedList->isColumnHidden(F_UPSPEED)) {
       finishedListModel->setData(finishedListModel->index(row, F_UPSPEED), QVariant((double)h.upload_payload_rate()));
     }
-    if(!finishedList->isColumnHidden(F_LEECH)) {
-      finishedListModel->setData(finishedListModel->index(row, F_LEECH), misc::toQString(h.num_peers() - h.num_seeds(), true));
+    if(!finishedList->isColumnHidden(F_PEERS)) {
+      finishedListModel->setData(finishedListModel->index(row, F_PEERS), misc::toQString(h.num_peers() - h.num_seeds(), true));
+    }
+    if(!finishedList->isColumnHidden(F_UPLOAD)) {
+      finishedListModel->setData(finishedListModel->index(row, F_UPLOAD), QVariant((double)h.all_time_upload()));
     }
     if(!finishedList->isColumnHidden(F_RATIO)) {
       finishedListModel->setData(finishedListModel->index(row, F_RATIO), QVariant(misc::toQString(BTSession->getRealRatio(hash))));
@@ -294,13 +367,13 @@ void FinishedTorrents::pauseTorrent(QString hash) {
     return;
   finishedListModel->setData(finishedListModel->index(row, F_UPSPEED), QVariant((double)0.0));
   finishedListModel->setData(finishedListModel->index(row, F_NAME), QIcon(QString::fromUtf8(":/Icons/skin/paused.png")), Qt::DecorationRole);
-  finishedListModel->setData(finishedListModel->index(row, F_LEECH), QVariant(QString::fromUtf8("0")));
+  finishedListModel->setData(finishedListModel->index(row, F_PEERS), QVariant(QString::fromUtf8("0")));
   setRowColor(row, QString::fromUtf8("red"));
 }
 
 QString FinishedTorrents::getHashFromRow(unsigned int row) const {
-  Q_ASSERT(row < (unsigned int)finishedListModel->rowCount());
-  return finishedListModel->data(finishedListModel->index(row, F_HASH)).toString();
+  Q_ASSERT(row < (unsigned int)proxyModel->rowCount());
+  return proxyModel->data(proxyModel->index(row, F_HASH)).toString();
 }
 
 // Will move it to download tab
@@ -317,7 +390,7 @@ void FinishedTorrents::deleteTorrent(QString hash){
 
 // Show torrent properties dialog
 void FinishedTorrents::showProperties(const QModelIndex &index){
-  showPropertiesFromHash(finishedListModel->data(finishedListModel->index(index.row(), F_HASH)).toString());
+  showPropertiesFromHash(getHashFromRow(index.row()));
 }
 
 void FinishedTorrents::showPropertiesFromHash(QString hash){
@@ -348,14 +421,15 @@ void FinishedTorrents::forceRecheck(){
   QModelIndexList selectedIndexes = finishedList->selectionModel()->selectedIndexes();
   foreach(const QModelIndex &index, selectedIndexes){
       if(index.column() == F_NAME){
-          QString hash = finishedListModel->data(finishedListModel->index(index.row(), F_HASH)).toString();
+          QString hash = getHashFromRow(index.row());
           QTorrentHandle h = BTSession->getTorrentHandle(hash);
+          qDebug("Forcing recheck for torrent %s", hash.toLocal8Bit().data());
           h.force_recheck();
       }
   }
 }
 
-void FinishedTorrents::displayFinishedListMenu(const QPoint& pos){
+void FinishedTorrents::displayFinishedListMenu(const QPoint&){
   QMenu myFinishedListMenu(this);
   // Enable/disable pause/start action given the DL state
   QModelIndexList selectedIndexes = finishedList->selectionModel()->selectedIndexes();
@@ -363,7 +437,7 @@ void FinishedTorrents::displayFinishedListMenu(const QPoint& pos){
   foreach(const QModelIndex &index, selectedIndexes) {
     if(index.column() == F_NAME) {
       // Get the file name
-      QString hash = finishedListModel->data(finishedListModel->index(index.row(), F_HASH)).toString();
+      QString hash = getHashFromRow(index.row());
       // Get handle and pause the torrent
       QTorrentHandle h = BTSession->getTorrentHandle(hash);
       if(!h.is_valid()) continue;
@@ -396,11 +470,11 @@ void FinishedTorrents::displayFinishedListMenu(const QPoint& pos){
   myFinishedListMenu.addAction(actionOpen_destination_folder);
   myFinishedListMenu.addAction(actionTorrent_Properties);
   myFinishedListMenu.addSeparator();
+  myFinishedListMenu.addAction(actionCopy_magnet_link);
   myFinishedListMenu.addAction(actionBuy_it);
 
   // Call menu
-  // XXX: why mapToGlobal() is not enough?
-  myFinishedListMenu.exec(mapToGlobal(pos)+QPoint(10,58));
+  myFinishedListMenu.exec(QCursor::pos());
 }
 
 
@@ -409,7 +483,7 @@ void FinishedTorrents::displayFinishedListMenu(const QPoint& pos){
  */
 
 // hide/show columns menu
-void FinishedTorrents::displayFinishedHoSMenu(const QPoint& pos){
+void FinishedTorrents::displayFinishedHoSMenu(const QPoint&){
   QMenu hideshowColumn(this);
   hideshowColumn.setTitle(tr("Hide or Show Column"));
   int lastCol = F_RATIO;
@@ -417,7 +491,7 @@ void FinishedTorrents::displayFinishedHoSMenu(const QPoint& pos){
     hideshowColumn.addAction(getActionHoSCol(i));
   }
   // Call menu
-  hideshowColumn.exec(mapToGlobal(pos)+QPoint(10,34));
+  hideshowColumn.exec(QCursor::pos());
 }
 
 // toggle hide/show a column
@@ -435,12 +509,12 @@ void FinishedTorrents::hideOrShowColumn(int index) {
     if(nbVisibleColumns <= 1) return;
     // User can hide the column, do it.
     finishedList->setColumnHidden(index, true);
-    getActionHoSCol(index)->setIcon(QIcon(QString::fromUtf8(":/Icons/button_cancel.png")));
+    getActionHoSCol(index)->setIcon(QIcon(QString::fromUtf8(":/Icons/oxygen/button_cancel.png")));
     --nbVisibleColumns;
   } else {
     // User want to display the column
     finishedList->setColumnHidden(index, false);
-    getActionHoSCol(index)->setIcon(QIcon(QString::fromUtf8(":/Icons/button_ok.png")));
+    getActionHoSCol(index)->setIcon(QIcon(QString::fromUtf8(":/Icons/oxygen/button_ok.png")));
     ++nbVisibleColumns;
   }
   //resize all others non-hidden columns
@@ -463,8 +537,16 @@ void FinishedTorrents::hideOrShowColumnUpSpeed() {
   hideOrShowColumn(F_UPSPEED);
 }
 
-void FinishedTorrents::hideOrShowColumnLeechers() {
-  hideOrShowColumn(F_LEECH);
+void FinishedTorrents::hideOrShowColumnSwarm() {
+  hideOrShowColumn(F_SWARM);
+}
+
+void FinishedTorrents::hideOrShowColumnPeers() {
+  hideOrShowColumn(F_PEERS);
+}
+
+void FinishedTorrents::hideOrShowColumnUpload() {
+  hideOrShowColumn(F_UPLOAD);
 }
 
 void FinishedTorrents::hideOrShowColumnRatio() {
@@ -490,9 +572,9 @@ bool FinishedTorrents::loadHiddenColumns() {
   for(int i=0; i<finishedListModel->columnCount()-1; i++) {
     if(loaded && ishidden_list.at(i) == "0") {
       finishedList->setColumnHidden(i, true);
-      getActionHoSCol(i)->setIcon(QIcon(QString::fromUtf8(":/Icons/button_cancel.png")));
+      getActionHoSCol(i)->setIcon(QIcon(QString::fromUtf8(":/Icons/oxygen/button_cancel.png")));
     } else {
-      getActionHoSCol(i)->setIcon(QIcon(QString::fromUtf8(":/Icons/button_ok.png")));
+      getActionHoSCol(i)->setIcon(QIcon(QString::fromUtf8(":/Icons/oxygen/button_ok.png")));
     }
   }
   return loaded;
@@ -526,8 +608,14 @@ QAction* FinishedTorrents::getActionHoSCol(int index) {
     case F_UPSPEED :
       return actionHOSColUpSpeed;
       break;
-    case F_LEECH :
-      return actionHOSColLeechers;
+    case F_SWARM :
+      return actionHOSColSwarm;
+      break;
+    case F_PEERS :
+      return actionHOSColPeers;
+      break;
+    case F_UPLOAD :
+      return actionHOSColUpload;
       break;
     case F_RATIO :
       return actionHOSColRatio;
@@ -536,94 +624,3 @@ QAction* FinishedTorrents::getActionHoSCol(int index) {
       return NULL;
   }
 }
-
-
-/*
- * Sorting functions
- */
-
-void FinishedTorrents::toggleFinishedListSortOrder(int index) {
-  Qt::SortOrder sortOrder = Qt::AscendingOrder;
-  if(finishedList->header()->sortIndicatorSection() == index){
-    sortOrder = (Qt::SortOrder)!(bool)finishedList->header()->sortIndicatorOrder();
-  }
-  switch(index) {
-    case F_SIZE:
-    case F_UPSPEED:
-    case F_RATIO:
-      sortFinishedListFloat(index, sortOrder);
-      break;
-    default:
-      sortFinishedListString(index, sortOrder);
-  }
-  QSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
-  QString sortOrderLetter;
-  if(sortOrder == Qt::AscendingOrder)
-    sortOrderLetter = QString::fromUtf8("a");
-  else
-    sortOrderLetter = QString::fromUtf8("d");
-  settings.setValue(QString::fromUtf8("FinishedListSortedCol"), misc::toQString(index)+sortOrderLetter);
-}
-
-void FinishedTorrents::sortFinishedList(int index, Qt::SortOrder sortOrder){
-  if(index == -1) {
-    index = finishedList->header()->sortIndicatorSection();
-    sortOrder = finishedList->header()->sortIndicatorOrder();
-  } else {
-    finishedList->header()->setSortIndicator(index, sortOrder);
-  }
-  switch(index) {
-    case F_SIZE:
-    case F_UPSPEED:
-      sortFinishedListFloat(index, sortOrder);
-      break;
-    default:
-      sortFinishedListString(index, sortOrder);
-  }
-}
-
-void FinishedTorrents::sortFinishedListFloat(int index, Qt::SortOrder sortOrder){
-  QList<QPair<int, double> > lines;
-  // insertion sorting
-  unsigned int nbRows = finishedListModel->rowCount();
-  for(unsigned int i=0; i<nbRows; ++i){
-    misc::insertSort(lines, QPair<int,double>(i, finishedListModel->data(finishedListModel->index(i, index)).toDouble()), sortOrder);
-  }
-  // Insert items in new model, in correct order
-  unsigned int nbRows_old = lines.size();
-  for(unsigned int row=0; row<nbRows_old; ++row){
-    finishedListModel->insertRow(finishedListModel->rowCount());
-    unsigned int sourceRow = lines[row].first;
-    unsigned int nbColumns = finishedListModel->columnCount();
-    for(unsigned int col=0; col<nbColumns; ++col){
-      finishedListModel->setData(finishedListModel->index(nbRows_old+row, col), finishedListModel->data(finishedListModel->index(sourceRow, col)));
-      finishedListModel->setData(finishedListModel->index(nbRows_old+row, col), finishedListModel->data(finishedListModel->index(sourceRow, col), Qt::DecorationRole), Qt::DecorationRole);
-      finishedListModel->setData(finishedListModel->index(nbRows_old+row, col), finishedListModel->data(finishedListModel->index(sourceRow, col), Qt::ForegroundRole), Qt::ForegroundRole);
-    }
-  }
-  // Remove old rows
-  finishedListModel->removeRows(0, nbRows_old);
-}
-
-void FinishedTorrents::sortFinishedListString(int index, Qt::SortOrder sortOrder){
-  QList<QPair<int, QString> > lines;
-  // Insertion sorting
-  unsigned int nbRows = finishedListModel->rowCount();
-  for(unsigned int i=0; i<nbRows; ++i){
-    misc::insertSortString(lines, QPair<int, QString>(i, finishedListModel->data(finishedListModel->index(i, index)).toString()), sortOrder);
-  }
-  // Insert items in new model, in correct order
-  unsigned int nbRows_old = lines.size();
-  for(unsigned int row=0; row<nbRows_old; ++row){
-    finishedListModel->insertRow(finishedListModel->rowCount());
-    unsigned int sourceRow = lines[row].first;
-    unsigned int nbColumns = finishedListModel->columnCount();
-    for(unsigned int col=0; col<nbColumns; ++col){
-      finishedListModel->setData(finishedListModel->index(nbRows_old+row, col), finishedListModel->data(finishedListModel->index(sourceRow, col)));
-      finishedListModel->setData(finishedListModel->index(nbRows_old+row, col), finishedListModel->data(finishedListModel->index(sourceRow, col), Qt::DecorationRole), Qt::DecorationRole);
-      finishedListModel->setData(finishedListModel->index(nbRows_old+row, col), finishedListModel->data(finishedListModel->index(sourceRow, col), Qt::ForegroundRole), Qt::ForegroundRole);
-    }
-  }
-  // Remove old rows
-  finishedListModel->removeRows(0, nbRows_old);
-}
